<?php
class sellsyHelpers {

	static public function retry($f, $delay = 1, $retries = 3) {

    try {
      return $f();
    }

    catch (Exception $e) {
      if ($retries > 0 && $e->getCode() == 429) {
        sleep($delay);
        echo 'retry !' . PHP_EOL;
        return self::retry($f, $delay, $retries - 1);
      } else {
        throw $e;

      }
    }
  }

	static public function runSellsy($request) {
		$checkApi = sellsyHelpers::checkApi();
		if (!$checkApi)
			return false;

		$sellsyCo = sellsyConnect_curl::load();
    	$response = self::retry(function() use ($sellsyCo, $request) { return $sellsyCo->requestApi($request); });

		if (!isset($response))
			return false;

		if ($response->status != 'success') {
			// Log!
			//echo "ERROR: " . $request['method'].  " " . print_r($response->error,true) . "\n";
			$fp = fopen(WOOCOMMERCE_SELLSY_PLUGIN_DIR . '/logs/error/' . date('Y-m') . '.txt','a+');
			fseek($fp,SEEK_END);
			$newlog  = "ERROR: " . $request['method'].  " " . print_r($response->error,true) . "\r\n";
			$newlog .= "RESPONSE/ " . print_r($response,true) . "\r\n";
			$newlog .= "REQUEST/ " . print_r($request,true) . "\r\n";
			fputs($fp,$newlog);
			fclose($fp);

			sellsyHelpers::log($response, $request);

			return $response;
		}

		if (isset($response->response))
      		return $response->response;

		return $response;
  	}


	static public function checkApi() {

    $request = array(
      'method' => 'Infos.getInfos',
      'params' => array(),

    );
    $sellsyCo = sellsyConnect_curl::load();
    $response = $sellsyCo->requestApi($request);
		return sellsyTools::storageGet('checkApi');
  }


	/*
   * Check if order is already in Sellsy account (by order id in custom field id-woocommerce)
   * @param $reference $order_id
   * @return false (if no) or proforma id (if yes)
   */

  static public function is_order_in_sellsy($order_id) {
    if ( !isset($GLOBALS['woo-ssy-custom-fields']) || empty($GLOBALS['woo-ssy-custom-fields']) ) {
      $GLOBALS['woo-ssy-custom-fields'] = sellsyHelpers::get_all_customfields();

    }

		$doctypes = get_option('woocommerce_sellsy_doctypes');
    $request = [
      'method' => 'Document.getList',
      'params' => [
          'doctype' => $doctypes[0],
          'search' => [
            'customfields' => [
              [
                'cfid'   => $GLOBALS['woo-ssy-custom-fields']['id-woocommerce'],
                'linkedtype'   => 'document',
                'value'   => [
                  'start' => $order_id,
                  'stop' => $order_id,
                ],
              ]
            ]
          ]
      ]
    ];
    $response = self::runSellsy($request);

		if ($response->infos->nbtotal < 1) {
        return false;
    }

		foreach ( $response->result as $r ) {
        if ( !$r->id) { continue; }
				return $r->id;
    }

		return false;
  }


	/*
   * Check if product is already in Sellsy account (by its name)
   * @param $reference varchar
   * @return false (if no) or item id (if yes)
   */

  static public function is_product_in_sellsy($reference, $type = 'item') {

    $request = [
      'method' => 'Catalogue.getList',
      'params' => [
          'type' => $type,
          'search' => [
            'name' => $reference
          ]
      ]
    ];
    $response = self::runSellsy($request);
		if (!$response) {
			return false;
		}

		foreach ( $response->result as $r ) {
        if ( !$r->id) { continue; }
				if ($r->name == $reference) {
            return $r;
        }
    }
  }


  // Catalogue
  static public function get_product($item_id) {

    $product = self::runSellsy([
      'method' => 'Catalogue.getOne',
      'params' => [
        'type' => 'item',
        'id' => $item_id
      ]
    ]);


		if (!empty($product->customfields)) {

      $product->cfs = array();
			foreach ($product->customfields as $cf) {

        $product->cfs[$cf->code] = '';
				if (isset($cf->formatted_value) && !empty($cf->formatted_value)) {
          $product->cfs[$cf->code] = $cf->formatted_value;
        }

        if (isset($cf->formattedVal) && !empty($cf->formattedVal)) {
          $product->cfs[$cf->code] = $cf->formattedVal;
        }

        if (isset($cf->raw_value) && isset($cf->formatted_value)) {
          $product->cfs[$cf->code] = [
            'id' => $cf->raw_value,
            'name' => $cf->formatted_value,
          ];
        }
      }
    }
		return $product;
  }


	static public function get_products($type = 'item') {

    $products = [];
    $searchService_r = [
      'method' => 'Catalogue.getList',
      'params' => [
        'type' => $type,
        'pagination' => [
          'pagenum'   => 1,
          'nbperpage' => '100',
        ],
      ]
    ];

    $searchService = self::runSellsy($searchService_r);

		foreach ( $searchService->result as $item )
    {
        if ( isset($item->id) && is_numeric($item->id) )
        {
            $products[$item->name] = $item;
        }
    }


		if ($searchService->infos->nbpages > 1) {

        for ($i=2; $i<=$searchService->infos->nbpages; $i++) {

          $searchService2_r = [

            'method' => 'Catalogue.getList',

            'params' => [

              'type' => "item",

              'pagination' => [
								'pagenum'   => $i,
								'nbperpage' => '100',

              ],

            ]

          ];

          $searchService2 = self::runSellsy($searchService2_r);


					foreach ( $searchService2->result as $item )
          {

              if ( isset($item->id) && is_numeric($item->id) )
              {
								  $products[$item->name] = $item;
              }
          }
        }
    }

		return $products;

  }


  /*
   * Create product in Sellsy Catalog
   * @param array datas from csv
   * @param item_id for update
   * @return item id

  static public function create_product($woo_item_id) {
     // $unitid = (isset($GLOBALS['units'][$datas['unite_de_vente']])) ? $GLOBALS['units'][$datas['unite_de_vente']] : false;
     $_pf = new WC_Product_Factory();
     $_product = $_pf->get_product($woo_item_id);
		 $taxrate = '20';

     if (!empty($_product)) {
       $tax_rates = WC_Tax::get_rates($_product->get_tax_class());
       if (!empty($tax_rates)) {
         $tax_rates = reset($tax_rates);
         $taxrate = $tax_rates['rate'];
       }
     }


     $categoryid = false;
     $product_categories_ids = $_product->get_category_ids();
     if (!empty($product_categories_ids)) {
        $full_category = get_term($product_categories_ids[0]);
        $category_name = $full_category->name;
        $categoryid = self::get_category($category_name);
     }

		 $is_already_in_ssy = self::is_product_in_sellsy($_product->get_sku());
		 $method = ($is_already_in_ssy) ? 'update' : 'create';

     $request = [
       'method' => 'Catalogue.' . $method,
       'params' => [
         'type' => 'item',
         'item' => [
            'name' => $_product->get_sku(),
            'tradename' => $_product->get_name(),
            'tradenametonote'   => 'N',
            'notes' => $_product->get_name() . ' ' . strip_tags($_product->get_description()),
            'unitAmount' => $_product->get_regular_price(),
            'unit' => 'unité',
            'qt' => 1,
            'unitAmountIsTaxesFree' => 'Y',
            'categoryid' => $categoryid,
            'taxrate' => $taxrate,
            'characteristics'   => array(
              'width' => $_product->get_width(),
              'length' => $_product->get_length(),
              'height' => $_product->get_height(),
              'weight' => $_product->get_weight(),
            )
         ]
       ]
     ];
     if ($is_already_in_ssy) {
       $request['params']['id'] = $is_already_in_ssy;
     }

		echo '<pre>';
		print_r($request);
		echo '</pre>';
		die();

     $response = self::runSellsy($request);
		 if (!$response) {
        return;
     }

		 $ssy_item_id = $response->item_id;

     // echo strtoupper($response->status) . " " . $method . " product " . $ssy_item_id . "\n";
		 //update_field('id_sellsy', $ssy_item_id, $is_already_in_ssy);

     update_post_meta($woo_item_id, '_id_sellsy', $ssy_item_id);


		 if ($method == 'create') {

		 		// Picture
        //$image = dirname(__FILE__) . '/files/product.jpg';
        $image = wp_get_attachment_image_src( get_post_thumbnail_id($woo_item_id), 'single-post-thumbnail' );

				if (!empty($image)) {

          $dir = wp_upload_dir();
          $image_path = str_replace(site_url(), '', $image[0]);
					$image_path = str_replace('/wp-content/uploads', $dir['basedir'], $image_path);
        }

        if (file_exists($image_path))
        {
            $addPicture_r = array(
								'method' => 'Catalogue.addPictureToGallery',
								'params' => array (
								  'id' => "$ssy_item_id",
								)
            );

            $requestFile = [
								"type" => mime_content_type($image_path),
								"name" => basename($image[0]),
								"tmp_name" => $image_path,
            ];

						$addPicture = sellsyConnect_curl::load()->requestApi($addPicture_r,$requestFile);
						/*
            if ($addPicture->status !== 'success')
            {
								echo "ERROR picture " . $service_id . "\n";
								echo "<pre>" . print_r($addPicture, true) . "</pre><pre>" . print_r($addPicture_r, true) . "</pre><pre>" . print_r($requestFile, true) . "</pre>";
            }
            * /
        }
     }
		 if ( !$ssy_item_id ) {
        return false;
     }

		 $cfs = self::save_CustomFields('item', $ssy_item_id, [
       ['cfid' => $GLOBALS['woo-ssy-custom-fields']['id-woocommerce'], 'value' => $woo_item_id],
     ]);

     // echo strtoupper($cfs->status) . " custom fields on product " . $ssy_item_id . " (Woo $woo_item_id)\n";

     if ( $cfs->status == 'error') {

        self::log($cfs, []);

        return;

     }


     return $ssy_item_id;
   }
   */



  /*
   * Get Sellsy third ID (prospect)
   * @param Sellsy Third ID
   * @return prospect object
   */

  static public function get_prospect($sellsy_id) {

    if (!$sellsy_id) { return false; }

		$getProspect_r =  [
        'method' => 'Prospects.getOne',
        'params' => [
          'id' => $sellsy_id
        ]
    ];

    $prospect = self::runSellsy($getProspect_r);

		if (!isset($prospect->customFields) || empty($prospect->customFields)) {
      return $prospect;

    }


		foreach ($prospect->customFields as $cfs) {

        foreach ($cfs->list as $cf) {
            $prospect->{$cf->code} = $cf->formatted_value;
        }
    }


		return $prospect;

  }




  /*
   * Get Sellsy third ID (client)
   * @param Sellsy Third ID
   * @return prospect object
   */
  static public function get_client($sellsy_id) {

    if (!$sellsy_id) { return false; }
			$getClient_r =  [
        'method' => 'Client.getOne',
        'params' => [
          'clientid' => $sellsy_id
        ]
    ];
    $client = self::runSellsy($getClient_r);

		if (!isset($client->customFields) || empty($client->customFields)) {
      return $client;
    }


		foreach ($client->customFields->nogroup->list as $cf) {
			if (!isset($cf->code)) { continue; }
			if (isset($cf->formatted_value)) {
        $client->{$cf->code} = $cf->formatted_value;
			}
    }

		return $client;
  }


	static public function search_client_by_mail($emailwoo) {

		$getClients_r = [
      'method' => 'Client.getList',
      'params' => [
        'search' => [
          'email' => $emailwoo
        ]
      ]
    ];

    $getClients = self::runSellsy($getClients_r);
		if ( $getClients->infos->nbtotal < 1 ) {
        return false;
    }


		foreach ( $getClients->result as $r ) {

        if ( $r->thirdid ) {
            return $r->thirdid;
        }
    }

		return false;

  }


			  static public function search_prospect_by_mail($emailwoo) {

    $request = [

      'method' => 'Prospects.getList',

      'params' => [

        'search' => [

          'email' => $emailwoo

        ]

      ]

    ];

    $response = self::runSellsy($request);


			    if ( $response->infos->nbtotal < 1 ) {

        return false;

    }


			    foreach ( $response->result as $r ) {

        if ( $r->thirdid ) {

            return $r->thirdid;

        }

    }


			    return false;

  }



  static public function create_guest($order_data) {

		$client_ssy_id = false;

    $addresses = [
      'billing' => [
        'company' => $order_data['billing']['company'],
        'first_name' => $order_data['billing']['first_name'],
        'last_name' => $order_data['billing']['last_name'],
        'address_1' => $order_data['billing']['address_1'],
        'address_2' => $order_data['billing']['address_2'],
        'postcode' => $order_data['billing']['postcode'],
        'city' => $order_data['billing']['city'],
        'countrycode' => $order_data['billing']['country'],
      ],

      'shipping' => [
        'company' => $order_data['shipping']['company'],
        'first_name' => $order_data['shipping']['first_name'],
        'last_name' => $order_data['shipping']['last_name'],
        'address_1' => $order_data['shipping']['address_1'],
        'address_2' => $order_data['shipping']['address_2'],
        'postcode' => $order_data['shipping']['postcode'],
        'city' => $order_data['shipping']['city'],
        'countrycode' => $order_data['shipping']['country'],

      ]
    ];

    $billing_name = (!empty($addresses['billing']['company'])) ? $addresses['billing']['company'] : $addresses['billing']['first_name'] . ' ' . $addresses['billing']['last_name'];
		$customer_type = (!empty($addresses['billing']['company'])) ? 'corporation' : 'person';

		$createThird_r = [
      'method' => 'Client.create',
      'params' => [
        'third' => [
          'name' => $billing_name,
          'type' => $customer_type,
          'email' => $order_data['billing']['email'],
          'tel' => $order_data['billing']['phone'],
          'tags' => 'WooCommerce',
        ],
      ]

    ];


		if ($customer_type == 'person') {

      $createThird_r['params']['contact'] = [
        'name' => $addresses['billing']['last_name'],
        'forename' => $addresses['billing']['first_name'],
        'email' => $order_data['billing']['email'],
        'tel' => $order_data['billing']['phone'],

      ];

    }

    if (isset($addresses['billing']) && !empty($addresses['billing']['address_1'])) {

      $createThird_r['params']['address'] = [
        'name' => $billing_name,
        'part1' => $addresses['billing']['address_1'],
        'part2' => $addresses['billing']['address_2'],
        'zip' => $addresses['billing']['postcode'],
        'town' => $addresses['billing']['city'],
        'countrycode' => $addresses['billing']['countrycode'],

      ];

    }

		$createThird = sellsyHelpers::runSellsy($createThird_r);

    if (!$createThird) {
      return false;

    }


		$client_ssy_id = $createThird->client_id;


		// Add Shipping address (if different than billing)

    if (isset($addresses['shipping']) && !empty($addresses['shipping']['address_1'])) {

      if ($addresses['billing']['address_1'] !== $addresses['shipping']['address_1']) {
        $shipping_name = (!empty($addresses['shipping']['company'])) ? $addresses['shipping']['company'] : $addresses['shipping']['first_name'] . ' ' . $addresses['shipping']['last_name'];
				$address_1 = $addresses['shipping']['address_1'];
				$address_2 = $addresses['shipping']['address_2'];
				$address_3 = '';

				if ($shipping_name !== $billing_name) {
					$address_1 = $shipping_name;
					$address_2 = $addresses['shipping']['address_1'];
					$address_3 = $addresses['shipping']['address_2'];
				}

        $createAddress_r =  [
            'method' => 'Addresses.create',
            'params' => [
								'linkedtype' => 'third',
								'linkedid'  => $client_ssy_id,
								'name' => $shipping_name,
								'part1' => $address_1,
								'part2' => $address_2,
								'part3' => $address_3,
								'zip' => $addresses['shipping']['postcode'],
								'town' => $addresses['shipping']['city'],
								'countrycode' => $addresses['shipping']['countrycode'],
        	      'isMainDeliv' => 'Y',
            ]
        ];
        $createAddress = sellsyHelpers::runSellsy($createAddress_r);

      }

    }


			    return $client_ssy_id;

  }


	// Create WooCommerce Client to Sellsy
  // @param WooCommerce $customer
  // @param third_type (client | prospect) following the number of orders
  static public function create_client($customer, $third_type = 'client') {
    $client_ssy_id = false;
    $method = ($third_type == 'client') ? 'Client' : 'Prospects';
    $customer_id = $customer->data->ID;
    $customer_type = false;
    $sync_roles = get_option('woocommerce_sellsy_roles');
    if (isset($customer->roles) && !empty($customer->roles)) {
      foreach ($customer->roles as $role) {
        if (in_array($role, $sync_roles)) {
          $customer_type = 'person';
          break;
        }
      }
    }

	if (!$customer_type) {
		echo "Customer type not selected in settings";
		return false;
    }


	$addresses = [
      'billing' => [
        'company' => get_user_meta($customer_id, 'billing_company', true),
        'first_name' => get_user_meta($customer_id, 'billing_first_name', true),
        'last_name' => get_user_meta($customer_id, 'billing_last_name', true),
        'address_1' => get_user_meta($customer_id, 'billing_address_1', true),
        'address_2' => get_user_meta($customer_id, 'billing_address_2', true),
        'postcode' => get_user_meta($customer_id, 'billing_postcode', true),
        'city' => get_user_meta($customer_id, 'billing_city', true),
        'countrycode' => get_user_meta($customer_id, 'billing_country', true),

      ],

      'shipping' => [
        'company' => get_user_meta($customer_id, 'shipping_company', true),
        'first_name' => get_user_meta($customer_id, 'shipping_first_name', true),
        'last_name' => get_user_meta($customer_id, 'shipping_last_name', true),
        'address_1' => get_user_meta($customer_id, 'shipping_address_1', true),
        'address_2' => get_user_meta($customer_id, 'shipping_address_2', true),
        'postcode' => get_user_meta($customer_id, 'shipping_postcode', true),
        'city' => get_user_meta($customer_id, 'shipping_city', true),
        'countrycode' => get_user_meta($customer_id, 'shipping_country', true),
      ]
    ];

		$joind = explode(' ', $customer->data->user_registered)[0];
    $exp_joind = explode('-', $joind);
    $joindate = strtotime($exp_joind[2] . '-' . $exp_joind[1] . '-' . $exp_joind[0]);

		$billing_name = (!empty($addresses['billing']['company'])) ? $addresses['billing']['company'] : $customer->data->display_name;

    $createThird_r = [
      'method' => $method . '.create',
      'params' => [
        'third' => [
          'name' => $billing_name,
          'type' => $customer_type,
          'email' => $customer->data->user_email,
          'tel' => get_user_meta($customer_id,'user_phone',true),
          'joinDate' => $joindate,
          'tags' => 'WooCommerce',
        ],
      ]
    ];

		if (empty($customer->last_name)) {
      $customer->last_name = $customer->data->display_name;
    }


		if (!empty($customer->last_name)) {

      $createThird_r['params']['contact'] = [
        'name' => $customer->last_name,
        'forename' => $customer->first_name,
        'email' => $customer->data->user_email,
      ];

    }

    if (isset($addresses['billing']) && !empty($addresses['billing']['address_1'])) {

      $billing_name = (!empty($addresses['billing']['company'])) ? $addresses['billing']['company'] : $addresses['billing']['first_name'] . ' ' . $addresses['billing']['last_name'];

      $createThird_r['params']['address'] = [
        'name' => $billing_name,
        'part1' => $addresses['billing']['address_1'],
        'part2' => $addresses['billing']['address_2'],
        'zip' => $addresses['billing']['postcode'],
        'town' => $addresses['billing']['city'],
        'countrycode' => $addresses['billing']['countrycode'],

      ];

    }

    $createThird = sellsyHelpers::runSellsy($createThird_r);


			    if (!$createThird) {

      return false;

    }


		$client_ssy_id = ($third_type == 'client') ? $createThird->client_id : $createThird;

    // echo "Third type : " . $third_type . "<br />";
    // pr($createThird);
    // echo "Client Sellsy ID/ " . $client_ssy_id . " <br />";

    if (!$client_ssy_id) {

      return;

    }


			    // Add Shipping address (if different than billing)

    if (isset($addresses['shipping']) && !empty($addresses['shipping']['address_1'])) {

      if ($addresses['billing']['address_1'] !== $addresses['shipping']['address_1']) {

        $shipping_name = (!empty($addresses['shipping']['company'])) ? $addresses['shipping']['company'] : $addresses['shipping']['first_name'] . ' ' . $addresses['shipping']['last_name'];
        $createAddress_r =  [

            'method' => 'Addresses.create',

            'params' => [
								'linkedtype' => 'third',
								'linkedid'  => $client_ssy_id,
								'name' => $shipping_name,
								'part1' => $addresses['shipping']['address_1'],
								'part2' => $addresses['shipping']['address_2'],
								'zip' => $addresses['shipping']['postcode'],
								'town' => $addresses['shipping']['city'],
								'countrycode' => $addresses['shipping']['countrycode'],

        	      'isMainDeliv' => 'Y',

            ]

        ];

        $createAddress = sellsyHelpers::runSellsy($createAddress_r);

      }

    }


			    // Update fields (emailwoo, woo id)

    $cfs = self::save_CustomFields($third_type, $client_ssy_id, [

      ['cfid' => $GLOBALS['woo-ssy-custom-fields']['id-woocommerce'], 'value' => $customer_id],

    ]);


			    return $client_ssy_id;

  }


	static public function createAddress($address, $client_ssy_id) {

    $address_name = (!empty($address['company'])) ? $address['company'] : $address['forename'] . ' ' . $address['name'];
    $createAddress_r =  [
        'method' => 'Addresses.create',
        'params' => [
            'linkedtype' => 'third',
            'linkedid'  => $client_ssy_id,
            'name' => $address_name,
            'part1' => $address['part1'],
            'part2' => $address['part2'],
            'zip' => $address['zip'],
            'town' => $address['town'],
            'countrycode' => $address['countrycode'],
            //'isMain' => 'N',
            //'isMainDeliv' => 'N'
        ]
    ];

    $createAddress = sellsyHelpers::runSellsy($createAddress_r);

		return $createAddress->id;

  }


	static public function transform_prospect($sellsy_id) {

    if (!$sellsy_id) { return false; }
			$transformProspect_r =  [
        'method' => 'Prospects.transformToCustomer',
        'params' => [
          'thirdid' => $sellsy_id,
          'enableCustomfieldsOnProspect' => true
        ]
    ];

    $transformProspect = self::runSellsy($transformProspect_r);


		return $sellsy_id;

  }


	static public function getPayMediums() {
			$mediums = [];
			$request = array(
			      'method' => 'Accountdatas.getPayMediums',
			      'params' => array()
			  );
			$response = sellsyHelpers::runSellsy($request);

			foreach ($response as $med) {
				$mediums[$med->value] = $med->id;
			}

			return $mediums;
	}

	static public function createPayment($payment) {

		$payMediums = self::getPayMediums();

    $createPayment_r = [
        'method' => 'Document.createPayment',
        'params' => [
            'payment' => [
				'date' => $payment['date'],
				'amount' => $payment['amount'],
				'medium' => $payMediums['carte bancaire'],
				'ident' => $payment['ident'],
				'notes' => '',
				'doctype' => $payment['doctype'],
				'docid' => $payment['docid'],

            ]
        ]
    ];

		return sellsyHelpers::runSellsy($createPayment_r);
  }


	/*
   * Get all custom fields, sorted by reference
   */

  static public function get_all_customfields() {
     $cfs = [];
     $request = [
     	'method' => 'CustomFields.getList',
     	'params' => [
     		'pagination'	=> [
     			'nbperpage'	=> '100',
     		],
     	]
     ];

     $response = self::runSellsy($request);


		 if (empty($response->result)) {

       return [];

     }

     foreach ($response->result as $k => $c) {

         if (!is_numeric($k)) {

             continue;

         }

         $cfs[$c->code] = $c->cfid;

     }


			     return $cfs;

   }


	 static public function get_shippings() {

      $shippings = [];
      $request = array(
        'method' =>  'Accountdatas.getShippingList',
        'params' => array()
      );
      $response = self::runSellsy($request);

			if (empty($response)) {
        return $shippings;
      }

			foreach ($response as $ship) {
				if (!isset($ship->isEnabled)) { continue; }
        if ($ship->isEnabled !== 'Y') { continue; }
        $shippings[$ship->id] = $ship->name;
      }

			return $shippings;
  }


	static public function get_langs() {

    $formattedLangs = [];
    $langs = self::runSellsy([
       'method' => 'Accountdatas.getTranslationLanguages',
       'params' => []
    ]);
		if (empty($langs)) {
      return [];
    }

		foreach ($langs as $lang) {
      $formattedLangs[$lang->id] = ($lang->flag == 'gb') ? 'en' : $lang->flag;
    }

		return $formattedLangs;
  }


	/*

   * Record custom fields values in Sellsy
   * @param type: item, client, prospect...
   * @param linkedin: item_id, third_id...
   * @param $datas = [
      * ['cfid' => cfid, 'value' => value],
      * ['cfid' => cfid, 'value' => value],

   * ]

   * @return array response, request

   */

  static public function save_CustomFields($type, $linkedid, $datas) {

       if (empty($datas)) { return false; }

       foreach ($datas as $k => $d) {

           if (empty($d['value'])) {

             unset($datas[$k]);

           }

       }


			 $request = [
       	'method' => 'CustomFields.recordValues',
       	'params' => array(
       		'linkedtype' 	=> $type,
       		'linkedid' 	=> $linkedid,
       		'values' 	=> $datas
       	)
       ];

       $response = self::runSellsy($request);

			 return $response;
  }


	static public function create_wooid_cf() {

    $request = array(
      'method' => 'CustomFields.create',
      'params' => array(
        'type'  => 'numeric',
        'name'  => 'ID WooCommerce',
        'code'  => 'id-woocommerce',
        'useOn' => array(
          'showIn_filter',
          'showIn_list',
          'useOn_document',
          'useOn_client',
          'useOn_item',
          'useOn_service'
        ),
        'preferences' => array(
          'isRequired'  => 'N',
          'description' => 'Ne pas toucher. Synchronisation de l\'extension WooCommerce to Sellsy',
        ),
      )
    );

    $response = self::runSellsy($request);
		if (isset($response->id)) {

      $GLOBALS['woo-ssy-custom-fields']['id-woocommerce'] = $response->id;

    }

  }


	static public function getDelivery($delivery_id) {

    $delivery = self::runSellsy([
        'method' => 'Document.getOne',
        'params' => array(
            'doctype' => 'delivery',
            'docid' => $delivery_id
        )
    ]);


		if (isset($delivery->customfieldsGroups)) {

        $delivery->customfields = [];

				foreach ($delivery->customfieldsGroups as $cf_group => $list)
        {
            if (!isset($list->list)) {
              continue;
            }
						foreach ($list->list as $cf) {
								$delivery->customfields[$cf->code] = $cf->formatted_value;
            }
        }
    }

    return $delivery;

  }


  /*
   * Get all units
   * @param version light (value only) || full (value + id)
   * @return array
   */

  static public function get_units($version = 'light') {
     $units = [];
     $request = array(
         'method' =>  'Accountdatas.getUnits',
         'params' => array()
     );

     $response = self::runSellsy($request);

     foreach ($response as $r) {
       if ($version == 'light') {
         $units[] = $r->value;
       }
			 if ($version == 'full') {
         $units[$r->value] = $r->id;
       }

     }
		 return $units;

   }

   /*
    * Get all taxes
    * @return array
    */
  static public function get_taxes() {
     $taxes = [];
     $request = [
     	'method' => 'Accountdatas.getTaxes',
     	'params' => []
     ];

     $response = sellsyHelpers::runSellsy($request);

		 foreach ($response as $t)
     {
         if (isset($t->id) && !empty($t->id))
         {
             $val = str_replace('.',',',round($t->value,1));
             $taxes["$val"] = $t->id;
         }
     }

		 return $taxes;
   }

   static public function get_accountings() {

      $accountings = [];

      $request = [
      	'method' => 'Accounting.getList',
      	'params' => [
          'pagination' 	=> [
			      'nbperpage' => '300'
		      ],
          'search' => [
            'view' => 'sell'
          ]
        ]
      ];

      $response = sellsyHelpers::runSellsy($request);

			foreach ($response as $t)
      {
          if (isset($t->id) && !empty($t->id))
          {
              $accountings[$t->code] = $t->id;
          }
      }

			return $accountings;
    }


			   /*

    * Get all product categories

    */

  static public function get_categories() {

      $categories = [];
      $request = array(
          'method' =>  'Catalogue.getCategories',
          'params' => array()
      );

			return self::runSellsy($request);
  }




   /*

    * Get Sellsy category ID from category level

    * @param WooCommerce category (Legumes>Pommes de terre)

    * @return Sellsy ID

    */

   static public function get_category($category_name){

	 			$ssy_categories = self::get_categories();

	 			foreach ($ssy_categories as $cat) {

	 				if ($cat->name == $category_name) {

							return $cat->id;
					}

					if (!empty($cat->children))
					{

            foreach ($cat->children as $child) {
								if ($child->name == $category_name) {
								    return $child->id;
								}

            }

        }
			}
			return false;
    }


		static public function get_staffs() {

      $staffs = array();


			$request =  array(
        'method' => 'Staffs.getList',
        'params' => array(
          'pagination' => array(
            'nbperpage' => "100",
            'pagenum' => "1"
          )
        )
      );

      $response = self::runSellsy($request);

			foreach ($response->result as $p) {

        if (!isset($p->id)) { continue; }
        if ($p->actif !== 'Y') { continue; }
        $staffs[str_replace('staff_', '', $p->id)] = $p->forename . ' ' . $p->name;
      }


			return $staffs;

   }


	 static public function log($response, $request) {

     $debugEmail = get_option('woocommerce_sellsy_debug_email');
     if (empty($debugEmail)) {
       return true;
     }

     if ($response->status == 'success') {
       return;
     }


		 // Send email
     $sendMail_r =  array(
       'method' => 'Mails.sendOne',
       'params' => array(
         'email'    => array(
           'emails' => [get_option('woocommerce_sellsy_debug_email')],
           'content' =>  "ERROR " . $request['method'] . ": " . $response->error->message . " <pre>" . print_r($response,true). "</pre> <pre>" . print_r($request,true). "</pre>\n",
           'subject' => "ERROR Woo woocommerce-to-sellsy (" . $request['method'] . ")",
           'includeDoc' => 'N',
         )
       )
     );
		 // $sendMail = self::runSellsy($sendMail_r);
   }
}
