<?php
class sellsyThirds {

  static public function getPrefs($third_id) {

    $getClient_r = [
      'method' => 'Client.getOne',
      'params' => [
          'clientid' => $third_id
      ]
    ];
    $getClient = sellsyHelpers::runSellsy($getClient_r);

    return $getClient->prefs;
  }

  static public function getThirdByEmail($email) {
    if (empty($email)) {
	    echo "Email empty<br />";
      return false;
    }

    if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
	    echo "Email not valid<br />";
      return false;
    }

    $third_type = 'client';
    $third_id = false;

    // Search client
    $third_id = sellsyHelpers::search_client_by_mail($email);
    if ($third_id) {
      return [
        'third_type' => $third_type,
        'third_id' => $third_id,
      ];
    }

    // Search prospect
    $third_id = sellsyHelpers::search_prospect_by_mail($email);
    if ($third_id) {
      $third_type = 'prospect';

      return [
        'third_type' => $third_type,
        'third_id' => $third_id,
      ];
    }
  }


  // Check if WooCommerce Customer is already in Sellsy account
  // @param WooCommerce Customer ID
  // @return Sellsy third id
  static public function checkThird($customer_id, $order_data = array()) {
	  // echo "Check third | Customer ID/ " . $customer_id;

	  $customer = false;
	  if (strstr($customer_id, '@')) {
		  $email = $customer_id;
		  $customer_id = false;
	  }
	  else {
    	$customer = get_userdata($customer_id);
		$email = $customer->data->user_email;
	  }

    	$ssy_third = self::getThirdByEmail($email);
		if ($customer_id) {
    	// Check (in woocommerce) if third has already ordered
			$countOrders = wc_get_customer_order_count($customer_id);
		}

    // Third ID exists in Sellsy AND it's a client > just return Sellsy id
    if ( $ssy_third['third_id'] && ($ssy_third['third_type'] == 'client') ) {
      // echo "Woo Customer " . $email . " already as client in Sellsy (" . $ssy_third['third_id'] . ")<br />";
      return $ssy_third['third_id'];
    }

  	if ( $ssy_third['third_id'] && ($ssy_third['third_type'] == 'prospect') ) {
      // echo "Woo Customer " . $email . " already as prospect in Sellsy (" . $ssy_third['third_id'] . ") (Nb orders: " . $countOrders . ")<br />";
	    // if he's already ordered > transform prospect to client

      if ($countOrders > 0) {
        // echo "Transform prospect to client (id " . $ssy_third['third_id'] . ")!<br />";
        return sellsyHelpers::transform_prospect($ssy_third['third_id']);
      }
  	}

  	if ( !$ssy_third['third_id'] ) {
  		// Create third
      if ($customer) {
     		return self::create_third($customer, $countOrders);
      }

      // Guest
		return sellsyHelpers::create_guest($order_data);
  	}
  }

  static public function create_third($customer, $countOrders) {
      // $third_type = ($countOrders > 0) ? 'client' : 'prospect';
      $third_type = 'client';

      return sellsyHelpers::create_client($customer, $third_type);
  }

  static public function import_wc_thirds_to_ssy() {
    if ( !isset($GLOBALS['woo-ssy-custom-fields']) || empty($GLOBALS['woo-ssy-custom-fields']) ) {
      $GLOBALS['woo-ssy-custom-fields'] = sellsyHelpers::get_all_customfields();
    }

    // Check if there are new files
    $hooksDir = WOOCOMMERCE_SELLSY_PLUGIN_DIR . '/hooks/files/';
    $files = scandir($hooksDir);
    if (empty($files)) {
      return false;
    }

    foreach ($files as $f) {
      if (strlen($f) < 5 ) {
        continue;
      }

      if (!strstr($f, 'user')) {
        continue;
      }

      // Move to exec
      rename($hooksDir . $f, WOOCOMMERCE_SELLSY_PLUGIN_DIR . '/hooks/exec/' . $f);

      // Get hook content
      $user_id = trim(file_get_contents(WOOCOMMERCE_SELLSY_PLUGIN_DIR . '/hooks/exec/' . $f));
      self::checkThird($user_id);

  		// Archive webhook file
  		unlink(WOOCOMMERCE_SELLSY_PLUGIN_DIR . '/hooks/exec/' . $f);
    }
  }
}
